// C Header File
// Created 1/1/01; 10:34:45 PM


void Animate();           //Interrupt driven firing, riding tankmovers, tunnels
void HaltAnimateTimer();   //Halts Animate Timer if it is appropriate to do so
void CheckAntiTank();            //Check to see if tank is in line of fire
void CheckDestination(int DestRow, int DestCol, char mode);  //Check where tank is going
void CheckForAction(int tr, int tc);   //See if animation must be started
void CheckForIce(int tr, int tc);  //See if animation needed for ice
void CrackIce(int CurRow, int CurCol);  //Turn ThinIce into water with UNDO
unsigned char ChooseLevelEntry(char *sptr);
void CursHor(int dir);           //Horizontal cursor handling
void CursVer(int dir);           //Vertical cursor handling
void DestroyCanon(char ShotsBit);             //Wipe out canon if hit with laser
void DisplayPalette();
void EnterTunnel(int CurRow,int CurCol);
void Fire();                     //Fire laser & start interrrup animation
char OnScreen(int RowPos, int ColPos);  //Check if position is on the screen
char OpenSpot(int SpriteId);     //Check if Id<=SunkenBlock or Id>=Tunnel
void ReflectLaser();             //Reflect laser off of mirrors
void RemoveBrick(char ShotsBit);              //Wipe out brick if hit with laser
void RotateMirror(char ShotsBit);             //Rotate mirrors if hit with laser
void SlideObject(int CurRow, int CurCol, int CurDir, char ShotsBit);              //Slide object if hit with laser
void TunnelExit(int *tr, int *tc);    //Handle tunnel animation
void Undo();
void UndoAdd(int row, int col, int id);
void UpdateStats(char direct);

void CursHor(int dir)
{
 HorInertia=dir;
 VerInertia=0;
 if (TankDn-dir != TankDir)
 {
  WriteSpritePlanes(TankRow,TankCol,board[TankRow][TankCol]);
  WriteSpritePlanes(TankRow,TankCol,TankDn-dir);
  TankDir=TankDn-dir;
  tx=134;
  WriteSpritePlanes(3,1,board[TankRow][TankCol]);
  WriteSpritePlanes(3,1,TankDir);
  tx=2;
  CheckAntiTank();
 }
 else
  if (TankCol+dir<16 && TankCol+dir>=0)
  {
   CheckDestination(TankRow,TankCol+dir,CursorMove);
   if (!Fatal)
   { 
   	CheckForIce(TankRow,TankCol);
   	CheckForAction(TankRow,TankCol);
   }
  }
}

void CursVer(int dir)
{
 HorInertia=0;
 VerInertia=dir;
 if (TankRt+dir != TankDir)
 {
  WriteSpritePlanes(TankRow,TankCol,board[TankRow][TankCol]);
  WriteSpritePlanes(TankRow,TankCol,TankRt+dir);
  TankDir=TankRt+dir;
  tx=134;
  WriteSpritePlanes(3,1,board[TankRow][TankCol]);
  WriteSpritePlanes(3,1,TankDir);
  tx=2;
  CheckAntiTank();
 }
 else
  if (TankRow+dir<12 && TankRow+dir>=0)
  {
   CheckDestination(TankRow+dir,TankCol,CursorMove);
   if (!Fatal) 
   {
    CheckForIce(TankRow,TankCol);
    CheckForAction(TankRow,TankCol);
   }
  }
}

void Fire()
{
 FireDir=TankDir-TankUp;
 FireRow=TankRow;
 FireCol=TankCol;
 BusyFire=FireLaser;
}

char OnScreen(int RowPos, int ColPos)
{
 return (RowPos<=11 && RowPos>=0 && ColPos<=15  && ColPos>=0);	
}

char OpenSpot(int SpriteId)
{
	return(SpriteId<=SunkenBlock || SpriteId>=Tunnel);
}

void CheckDestination(int DestRow,int DestCol,char mode)
{
 if (OnScreen(DestRow,DestCol) && OpenSpot(TopSprite(board[DestRow][DestCol])))
 {
  if (mode==CursorMove)
   UndoAdd(TankRow+64+128,TankCol,TankDir); //64 => Tank Moved  128 => single move undo
  WriteSpritePlanes(TankRow,TankCol,BottomSprite(board[TankRow][TankCol])); //Draw background
  WriteSpritePlanes(TankRow,TankCol,TopSprite(board[TankRow][TankCol])); //Draw Top Sprite
  TankRow=DestRow;
  TankCol=DestCol;
  WriteSpritePlanes(TankRow,TankCol,TankDir); //Draw tank appropriate direction
  switch (board[TankRow][TankCol])
  {
  	case Water:
  	 Fatal=Sunk; break;
  	case Flag:
  	 Fatal=GotFlag;
  }
  if(!BusyFire) CheckAntiTank();
 }
 else
 {
  BusyTank=False;
  Backtrack=False;
  TankInTunnel=False;
  if (board[TankRow][TankCol]==Water)  
   Fatal=CrackedIce;   //CrackedIce? Yes, stopping on ThinIce cracks ice: makes water
 }
}

void CheckForAction(int tr,int tc)
{
 if (board[tr][tc]>=TankMoverUp && board[tr][tc]<=TankMoverLt)
  BusyTank=True;
 if (board[tr][tc]>=80 && board[tr][tc]<=99)
  BusyTank=True;
}

void CheckForIce(int tr,int tc)
{
 if (board[tr][tc]==Ice || board[tr][tc]==ThinIce)
  BusyTank=True;
}

int BlockedOrInLaser(int r, int c, int direction)
{
 int truth;
 truth = False;
 if (TopSprite(board[r][c]==direction))
  truth=True||truth;
 if (TopSprite(board[r][c])>SunkenBlock && TopSprite(board[r][c])<Tunnel)
  truth=True||truth;
 return truth;
}

void CheckAntiTank()
{
 int r,c,fired;
 fired=False;
 r=TankRow;
 if (TopSprite(board[TankRow][TankCol])<CanonUp || TopSprite(board[TankRow][TankCol])>SlideMirrorUL)
 {
 for (c=TankCol+1;c<=15;c++)
  if (BlockedOrInLaser(r,c,CanonLt))
  {
   if (TopSprite(board[r][c])==CanonLt)
   {
    if (board[TankRow][TankCol]!=Flag)
    {
     BusyFire=AntiLaser; FireRow=r; FireCol=c-1; FireDir=3; fired=True;
    } 
    else
    {
     if (c-TankCol<2)
     {
      BusyFire=AntiLaser; FireRow=r; FireCol=c-1; FireDir=3; fired=True; Fatal=False;
     }
    }
   }
   break;
  }
 if (!fired)
  for (c=TankCol-1;c>=0;c--)
   if (BlockedOrInLaser(r,c,CanonRt))
   {
    if (TopSprite(board[r][c])==CanonRt)
    {
     if (board[TankRow][TankCol]!=Flag)
     {      
      BusyFire=AntiLaser; FireRow=r; FireCol=c+1; FireDir=1; fired=True;
     }
     else
     {
      if (TankCol-c<2)
      {
       BusyFire=AntiLaser; FireRow=r; FireCol=c+1; FireDir=1; fired=True; Fatal=False;
      }
     }
    }
    break; 
   }
   c=TankCol;
   if (!fired)
    for (r=TankRow+1;r<=11;r++)
     if (BlockedOrInLaser(r,c,CanonUp))
     {
      if (TopSprite(board[r][c])==CanonUp)
      {
       if (board[TankRow][TankCol]!=Flag)
       {
     BusyFire=AntiLaser; FireRow=r-1; FireCol=c; FireDir=0; fired=True;
       }
       else
       {
      if (r-TankRow<2)
        {
       BusyFire=AntiLaser; FireRow=r-1; FireCol=c; FireDir=0; fired=True; Fatal=False;
      }
       }
      }
      break;
     }
     if (!fired)
      for (r=TankRow-1;r>=0;r--)
       if (BlockedOrInLaser(r,c,CanonDn))
       {  
        if (TopSprite(board[r][c])==CanonDn)
        {
  if (board[TankRow][TankCol]!=Flag)
         {
    BusyFire=AntiLaser; FireRow=r+1; FireCol=c; FireDir=2; fired=True;
   }
   else
   {
    if (TankRow-r<2)
          {
      BusyFire=AntiLaser; FireRow=r+1; FireCol=c; FireDir=2; fired=True; Fatal=False;
     }
   }
        }  
 break;
       }
      if (fired && FireRow==TankRow && FireCol==TankCol) Fatal=Shot;
 }
}

void EnterTunnel(int CurRow, int CurCol)
{
 if (TankInTunnel)
 {
 	TankInTunnel=False;
  if (OnScreen(CurRow+VerInertia,CurCol+HorInertia))
 	{
		CheckDestination(CurRow+VerInertia,CurCol+HorInertia,0);
 		if (board[TankRow][TankCol]>=80 && board[TankRow][TankCol]<=99 && CurRow==TankRow && CurCol==TankCol)
 		 {
 		  Backtrack=True;
 		  BusyTank=True;
 		 }
 		else
 		{
 			if (!Fatal)
 			{
 				CheckForIce(TankRow,TankCol);
 				CheckForAction(TankRow,TankCol);
 			}
 		}
 	}
 	else
 	 Backtrack=True;
 }
 else
 {
 	TankInTunnel=True;
 	if (Backtrack)
 	{
 		HorInertia=-DeltaCol[TankDir-TankUp];
 		VerInertia=-DeltaRow[TankDir-TankUp];
 	}
 	else
 	{
		 HorInertia=DeltaCol[TankDir-TankUp];
		 VerInertia=DeltaRow[TankDir-TankUp];
		}
  TankRow=CurRow;
 	TankCol=CurCol;
  WriteSpritePlanes(TankRow,TankCol,board[TankRow][TankCol]);
 	TunnelExit(&TankRow,&TankCol);
 	WriteSpritePlanes(TankRow,TankCol,TankDir);
  if (!BusyFire)
   CheckAntiTank();
  } 
}

void TunnelExit(int *tr, int *tc)
{
 int DestRow,DestCol;
 if (Tpair[BottomSprite(board[*tr][*tc])-80][0]==*tr && Tpair[BottomSprite(board[*tr][*tc])-80][1]==*tc)
 {
  DestRow=Tpair[BottomSprite(board[*tr][*tc])-80][2];
  DestCol=Tpair[BottomSprite(board[*tr][*tc])-80][3];
 }
 else
 {
  DestRow=Tpair[BottomSprite(board[*tr][*tc])-80][0];
  DestCol=Tpair[BottomSprite(board[*tr][*tc])-80][1];
 }
 *tr=DestRow;
 *tc=DestCol;
}

void SlideObject(int CurRow, int CurCol, int CurDir, char ShotsBit)
{
 int NewRow,NewCol,SpriteId,Changed,TempSprite;
 Changed=False;
 SpriteId=BottomSprite(board[CurRow][CurCol]);
 if (SpriteId>=80 && SpriteId<=99 && !ObjectInTunnel)
 {
 	NewRow=CurRow;
 	NewCol=CurCol;
 	ObjectInTunnel=True;
 	TunnelExit(&NewRow,&NewCol);
 }
 else
 {
  NewRow=CurRow+DeltaRow[CurDir];
  NewCol=CurCol+DeltaCol[CurDir];
  ObjectInTunnel=False;
 }
 SpriteId=TopSprite(board[NewRow][NewCol]);
 if (OnScreen(NewRow,NewCol) && OpenSpot(SpriteId))
 {
  if (SpriteId==Ice || SpriteId==ThinIce || SpriteId==Tunnel)
  {
  	if (BusyMove==False)
    UndoAdd(CurRow+128,CurCol+ShotsBit,board[CurRow][CurCol]);
  	MoveRow=NewRow;
  	MoveCol=NewCol;
  	MoveDir=CurDir;
  	BusyMove=True;
  }
  else
  {
   Changed=BusyMove;
   BusyMove=False;
  }
  if (!BusyMove)
  {
   if (!Changed) UndoAdd(CurRow,CurCol+ShotsBit,board[CurRow][CurCol]);
   UndoAdd(NewRow,NewCol,board[NewRow][NewCol]);
  }
  TempSprite=BottomSprite(board[CurRow][CurCol]);
  board[NewRow][NewCol]=StackSprite(BottomSprite(board[NewRow][NewCol]),TopSprite(board[CurRow][CurCol]));
  board[CurRow][CurCol]=TempSprite;
  if (board[CurRow][CurCol]==ThinIce)
   CrackIce(CurRow,CurCol);
  WriteSpritePlanes(CurRow,CurCol,board[CurRow][CurCol]);
  WriteSpritePlanes(NewRow,NewCol,board[NewRow][NewCol]);
  if (CurRow==TankRow && CurCol==TankCol)
  { 
    if (!PiggyBack) UndoAdd(TankRow+64+128,TankCol,TankDir);
    PiggyBack = True;
  	TankRow=NewRow;
  	TankCol=NewCol;
  }
  if (NewRow==TankRow && NewCol==TankCol)
   	WriteSpritePlanes(TankRow,TankCol,TankDir);
  else
  {
   if (board[TankRow][TankCol]<=Flag)
   {
    if (!BusyFire) CheckAntiTank();
    if (Fatal) 
    { 
    	board[NewRow][NewCol]=BottomSprite(board[NewRow][NewCol]);
    	WriteSpritePlanes(NewRow,NewCol,board[NewRow][NewCol]);
    }
    else
     CheckForAction(TankRow,TankCol);
   }
  }
 }   
 else
 {
 	if (BottomSprite(board[CurRow][CurCol])>=80 && BottomSprite(board[CurRow][CurCol])<=99)
 	{
 		MoveDir+=2;
 		if (MoveDir>3) MoveDir=MoveDir-4;
  }
  else
  {
 	 Changed=BusyMove;
 	 BusyMove=False;
   if (BottomSprite(board[CurRow][CurCol])==ThinIce)
   {
   	UndoAdd(CurRow+128,CurCol,ThinIce);
   	board[CurRow][CurCol]=StackSprite(Water,TopSprite(board[CurRow][CurCol]));
   	WriteSpritePlanes(CurRow,CurCol,board[CurRow][CurCol]);
   }
   else
    if (Changed) UndoAdd(CurRow+128,CurCol,Ice);
   WriteSpritePlanes(TankRow,TankCol,TankDir);
   if (BottomSprite(board[TankRow][TankCol])==Water)
    Fatal=Sunk;
   else
   {
   	if (!BusyFire) CheckAntiTank();
   	if (!Fatal) CheckForAction(TankRow,TankCol);
   }
  }
 }
}

void ReflectLaser()
{
 int mirror;
 mirror = TopSprite(board[FireRow][FireCol]);
 if (mirror<19)
  mirror = mirror-SlideMirrorUR;
 else
  mirror = mirror-RotateMirrorUR;
 FireDir = Reflector[mirror][FireDir];
 WriteSpritePlanes(FireRow,FireCol,mirror+LaserUR);
}

void RotateMirror(char ShotsBit)
{
 UndoAdd(FireRow+128,FireCol+ShotsBit,board[FireRow][FireCol]); //128 => single move undo
 board[FireRow][FireCol]++;
 if (board[FireRow][FireCol]>RotateMirrorUL)
  board[FireRow][FireCol]=RotateMirrorUR;
 WriteSpritePlanes(FireRow,FireCol,Background);
 WriteSpritePlanes(FireRow,FireCol,board[FireRow][FireCol]);
} 

void DestroyCanon(char ShotsBit)
{
 UndoAdd(FireRow+128,FireCol+ShotsBit,board[FireRow][FireCol]); //128 => single move undo
 board[FireRow][FireCol]=DestroyedCanon;
 WriteSpritePlanes(FireRow,FireCol,board[FireRow][FireCol]);
}

void RemoveBrick(char ShotsBit)
{
 UndoAdd(FireRow+128,FireCol+ShotsBit,board[FireRow][FireCol]); //128 => single move undo
 board[FireRow][FireCol]=Background;
 WriteSpritePlanes(FireRow,FireCol,board[FireRow][FireCol]);
 CheckAntiTank();
 if (!Fatal) CheckForAction(TankRow,TankCol);
}

void CrackIce(int CurRow, int CurCol)
{
 UndoAdd(CurRow+128,CurCol,ThinIce);
  board[CurRow][CurCol]=Water;
}

void HaltAnimateTimer()
{
 if (!BusyTank && !BusyFire)
  OSVFreeTimer(2);
}

void Animate()
{
 int SpriteID;
 char ShotsBit;
 char mode;
 ShotsBit=0;
 mode=0;
 SpriteID = TopSprite(board[FireRow][FireCol]);
 if (!BusyMove) PiggyBack=False;
 if (BusyFire)
 {
  if (FireCol!=TankCol || FireRow!=TankRow)
  {
   if (SpriteID>=SlideMirrorUR && SpriteID<=RotateMirrorUL)
    WriteSpritePlanes(FireRow,FireCol,BottomSprite(board[FireRow][FireCol]));
   WriteSpritePlanes(FireRow,FireCol,SpriteID);
  }
  else
   if (BusyFire==FireLaser)  
    BusyFire=TankLaser;
   else
    Fatal=Shot;
  FireCol=FireCol+DeltaCol[FireDir];
  FireRow=FireRow+DeltaRow[FireDir];
  if (FireCol < 0 || FireRow < 0 || FireRow > 11 || FireCol > 15)
  {
   BusyFire=False;
   CheckAntiTank(); 
  }
  else
  {
   if (TankRow==FireRow && TankCol==FireCol)
    Fatal=Shot;
   else
   {
    if (BusyFire==TankLaser) ShotsBit=64;
    switch (FireAction[TopSprite(board[FireRow][FireCol])][FireDir])
    {
     case 0: 
      WriteSpritePlanes(FireRow,FireCol,FireSprite[FireDir]); break;
     case 1:
      BusyFire=False; 
      break;
     case 2:
       if (BusyFire==TankLaser) mode=1;
       BusyFire=False; 
       if (!BusyMove)
        SlideObject(FireRow,FireCol,FireDir,ShotsBit);
      break;
     case 3:
      ReflectLaser(); break;
     case 4:
      BusyFire=False; RotateMirror(ShotsBit); 
      break;
     case 5:
      BusyFire=False; DestroyCanon(ShotsBit);
      break;
     case 6: 
      BusyFire=False; RemoveBrick(ShotsBit);
      break;
     }
    }
   }
 }
 if (BusyTank)
 {
  if (board[TankRow][TankCol]>=80 && board[TankRow][TankCol]<=99)
   EnterTunnel(TankRow,TankCol);
  else
   switch (TopSprite(board[TankRow][TankCol]))
   {
    case TankMoverUp:
     CheckDestination(TankRow-1,TankCol,mode);HorInertia=0;VerInertia=-1;break;
    case TankMoverRt:
     CheckDestination(TankRow,TankCol+1,mode);HorInertia=1;VerInertia=0;break;
    case TankMoverDn:
     CheckDestination(TankRow+1,TankCol,mode);HorInertia=0;VerInertia=1;break;
    case TankMoverLt:
     CheckDestination(TankRow,TankCol-1,mode);HorInertia=-1;VerInertia=0;break;
    case Ice:
     CheckDestination(TankRow+VerInertia,TankCol+HorInertia,mode);break;
    case ThinIce:
     CrackIce(TankRow,TankCol); CheckDestination(TankRow+VerInertia,TankCol+HorInertia,mode);break;
    default:
     BusyTank=False;Backtrack=False;TankInTunnel=False;
   }   
 }
 if (BusyMove)
 {
  if (BottomSprite(board[MoveRow][MoveCol])>=80 && BottomSprite(board[MoveRow][MoveCol])<=99)
   SlideObject(MoveRow,MoveCol,MoveDir,ShotsBit);
  else 
   switch (BottomSprite(board[MoveRow][MoveCol]))
   {
   	case Ice: 
   	 SlideObject(MoveRow,MoveCol,MoveDir,ShotsBit); break;
   	case ThinIce:
    	SlideObject(MoveRow,MoveCol,MoveDir,ShotsBit); break;
   	default:
   	 BusyMove=False;
   	 PiggyBack=False;
   }
 }
}
 
void UpdateStats(char direct)
{
 char outstr1[10];
 char outstr2[10];
 char outblnk[5]= {"    \0"};
 unsigned char xp1,xp2;
 FontSetSys(F_6x8);
 xp1=134;
 xp2=134;
 sprintf(outstr1,"%u",Shots);
 sprintf(outstr2,"%u",Moves);
 if (Shots<10)
  xp1=143;
 else
 { 
  if (Shots<100)
   xp1=140;
  else
  {
   if (Shots<1000)
    xp1=137;
   else
    xp1=134;
  }
 }
 if (Moves<10)
  xp2=143;
 else
 { 
  if (Moves<100)
   xp2=140;
  else
  {
   if (Moves<1000)
    xp2=137;
   else
    xp2=134;
  }
 }
 if (direct==Down && (Shots==9 || Shots==99 || Shots==999))
  WriteTextPlanes(134,45,outblnk,A_REPLACE);
 WriteTextPlanes(xp1,45,outstr1,A_REPLACE);
 if (direct==Down && (Moves==9 || Moves==99 || Moves==999))
  WriteTextPlanes(134,63,outblnk,A_REPLACE);
 WriteTextPlanes(xp2,63,outstr2,A_REPLACE);
}

// The 6th bit of undoRow keeps record of moves per undo
// The 7th bit of undoRow set indicates a tank position undo
// The 8th bit of undoRow clear indicates a double move undo from sliding
// The 7th bit of undoCol set indicates shots counter incremented
void UndoAdd(int row, int col, int id)
{
 unsigned char *ptr1,*ptr2;
 if (NewTurnOK & !BusyMove) play=32-play;
 NewTurnOK=False;
 if (BusyMove & (*undoRow & 32) != play) 
 {
 	 if (play = 32) 
 	   *undoRow = *undoRow + 32;
 	 else 
 	   *undoRow = *undoRow - 32; 
 }
 ptr1=undoRow;
 ptr2=undoRow+1;
 if ((undoRow[UNDOSIZE-2] & 128)==0) //Sliding objects requre double storage because it must
 {                                   //keep track of the sliding object and the background.
  undoRow[UNDOSIZE-2]=0xFF;          //These data items have the 128 bit set. If a slide object
  undoRow[UNDOSIZE-3]=0xFF;          //move shows up at the end of the queue, then pull out
  undoCol[UNDOSIZE-2]=0xFF;          //both data items.  This keeps the undo queue
  undoCol[UNDOSIZE-3]=0xFF;          //from undoing only half of a slide which essentially
 }                                   //loses the item that is being undone.
 if (64 & row) Moves+=1;
 if (64 & col) Shots+=1;
 memmove(ptr2,ptr1,UNDOSIZE-2);
 ptr1=undoCol;
 ptr2=undoCol+1;
 memmove(ptr2,ptr1,UNDOSIZE-2);
 ptr1=undoItem;
 ptr2=undoItem+1;
 memmove(ptr2,ptr1,UNDOSIZE-2);
 *undoRow = (unsigned char) row+play;
 *undoCol = (unsigned char) col;
 *undoItem = (unsigned char) id;
 if (64 & (*undoRow | *undoCol)) UpdateStats(Up);
}

void Undo()
{
 unsigned char *ptr1,*ptr2;
 int CurCol, CurRow;
 CurCol = -1;
 CurRow = -1;
 while ((*undoRow<0xFF) && (32 & *undoRow) == play)
 {
  if (64 & *undoRow)
  {
   WriteSpritePlanes(TankRow,TankCol,Background);
   WriteSpritePlanes(TankRow,TankCol,BottomSprite(board[TankRow][TankCol]));
   WriteSpritePlanes(TankRow,TankCol,TopSprite(board[TankRow][TankCol]));
   TankRow=*undoRow&31;
   TankCol=*undoCol&31;
   TankDir=*undoItem;
   WriteSpritePlanes(TankRow,TankCol,TankDir);
   Moves-=1;
  }
  else
  {
   CurRow = *undoRow&31;
   CurCol = *undoCol&31;
   WriteSpritePlanes(*undoRow&31,*undoCol&31,BottomSprite(*undoItem));
   WriteSpritePlanes(*undoRow&31,*undoCol&31,TopSprite(*undoItem));
   board[*undoRow&31][*undoCol&31]=*undoItem;
   WriteSpritePlanes(TankRow,TankCol,TankDir);
  }
  if (64 & *undoCol) Shots-=1;
  if (64 & (*undoRow | *undoCol)) UpdateStats(Down);
  ptr1=undoRow+1;
  ptr2=undoRow;
  memmove(ptr2,ptr1,UNDOSIZE-1);
  ptr1=undoCol+1;
  ptr2=undoCol;
  memmove(ptr2,ptr1,UNDOSIZE-1);
  ptr1=undoItem+1;
  ptr2=undoItem;
  memmove(ptr2,ptr1,UNDOSIZE-1);
 }
 play=32-play;
 BusyTank=False;
 Backtrack=False;
 if (BusyMove)
 {
 	BusyMove=False;
 	if ((CurRow != MoveRow) || (CurCol != MoveCol))
 		board[MoveRow][MoveCol] = BottomSprite(board[MoveRow][MoveCol]);
 	WriteSpritePlanes(MoveRow,MoveCol,BottomSprite(board[MoveRow][MoveCol]));
 	WriteSpritePlanes(MoveRow,MoveCol,TopSprite(board[MoveRow][MoveCol]));
 }
 	
 if (BusyFire)
 {
  BusyFire=False;
  WriteSpritePlanes(FireRow,FireCol,BottomSprite(board[FireRow][FireCol]));
  WriteSpritePlanes(FireRow,FireCol,TopSprite(board[FireRow][FireCol]));
 }
}
